#!/bin/bash
set -eu -o pipefail
cd `dirname $0`

source conts-env.sh

# iterate through each service required for integration testing
service_names=$(jq -r '.[].name' ${rspec_file})
for service_name in $service_names; do
    if ! lxc-info -n ${service_name}; then
        # create a container with a static ip address
        echo "Creating container: ${service_name}"
        SERVICE=$(jq --arg NAME ${service_name} '.[] | select(.name==$NAME)' ${rspec_file})
        echo $SERVICE
        ip=$(echo $SERVICE | jq -r '.ip_address')
        echo "dhcp-host=${service_name},${ip}" >> /etc/lxc/dnsmasq.conf
        service lxc-net restart

        lxc-create -t download -n ${service_name} -- --dist ubuntu --release xenial --arch amd64

        # copy flame clmc files into the root container
        echo "Copying files to rootfs"
        container_dir="/var/lib/lxc/"${service_name}"/rootfs"
        container_vagrant_dir=${container_dir}"${repo_root}"
        mkdir -p ${container_vagrant_dir}
        cp -f ${repo_root}/reporc "${container_vagrant_dir}"
        cp -rf ${repo_root}/scripts ${container_vagrant_dir}
        cp -rf ${repo_root}/src ${container_vagrant_dir}

        # start the container
        echo "Starting: ${service_name}"        
        lxc-start -n ${service_name}
        echo "Waiting for container to start: ${service_name}"
        STARTED="0"        
        while [ "$STARTED" == "0" ]; do STARTED=$(lxc-info -n ${service_name} -i | wc -l); done; 

        # provision software into each container
        echo "Provisioning: ${service_name}"
        if [ ${service_name} == "clmc-service" ]
        then
            influxdb_url=$(echo $SERVICE | jq -r '.influxdb_url')
            database_name=$(echo $SERVICE | jq -r '.database_name')
            report_period=$(echo $SERVICE | jq -r '.report_period')
            cmd="${repo_root}/scripts/clmc-service/install.sh ${influxdb_url} ${database_name} ${report_period}"
            lxc-attach -n ${service_name} -v REPO_ROOT=${repo_root} -- ${cmd}
        elif [ ${service_name} == "test-runner" ]
        then
            cmd=${repo_root}/src/test/clmctest/services/pytest/install.sh
            lxc-attach -n ${service_name} -- ${cmd}
        else
            # get container parameters
            location=$(echo $SERVICE | jq -r '.location')
            sf_id=$(echo $SERVICE | jq -r '.sf_id')
            sf_id_instance=$(echo $SERVICE | jq -r '.sf_id_instance')
            sfc_id=$(echo $SERVICE | jq -r '.sfc_id')
            sfc_id_instance=$(echo $SERVICE | jq -r '.sfc_id_instance')
            sr_id=$(echo $SERVICE | jq -r '.sr_id')
            ipendpoint_id=$(echo $SERVICE | jq -r '.ipendpoint_id')
            influxdb_url=$(echo $SERVICE | jq -r '.influxdb_url')
            database_name=$(echo $SERVICE | jq -r '.database_name')

            # install service function specific software
            cmd=${repo_root}/src/test/clmctest/services/${sf_id}/install.sh
            lxc-attach -n ${service_name} -v REPO_ROOT=${repo_root} -- ${cmd}

            # install telegraf
            cmd=${repo_root}/scripts/clmc-agent/install.sh
            lxc-attach -n ${service_name} -v REPO_ROOT=${repo_root} -- ${cmd}

            # copy telegraf configuration templates
            cp -f ${repo_root}/scripts/clmc-agent/telegraf.conf ${container_dir}/etc/telegraf/
            cp -f ${repo_root}/scripts/clmc-agent/telegraf_output.conf ${container_dir}/etc/telegraf/telegraf.d/
            cp ${repo_root}/src/test/clmctest/services/${sf_id}/telegraf_${sf_id}.conf ${container_dir}/etc/telegraf/telegraf.d/

            # replace telegraf template with container parameters
            # @todo do we really need both scripts to do this?
            cmd=${repo_root}/scripts/clmc-agent/configure_template.sh
            lxc-attach -n ${service_name} -- ${cmd}
            cmd="${repo_root}/scripts/clmc-agent/configure.sh ${location} ${sfc_id} ${sfc_id_instance} ${sf_id} ${sf_id_instance} ${ipendpoint_id} ${sr_id} ${influxdb_url} ${database_name}"
            lxc-attach -n ${service_name} -- ${cmd}

            # start telegraf
            lxc-attach -n ${service_name} -- service telegraf restart
        fi
    fi
done

#./conts-stop.sh

#service lxc-net restart

#./conts-start.sh